@@ -23,6 +23,7 @@ import (
 
 	"github.com/go-kit/log"
 	"github.com/go-kit/log/level"
+	"github.com/go-openapi/analysis"
 	"github.com/go-openapi/loads"
 	"github.com/go-openapi/runtime/middleware"
 	"github.com/go-openapi/strfmt"
@@ -73,9 +74,11 @@ type API struct {
 	Handler http.Handler
 }
 
-type groupsFn func(func(*dispatch.Route) bool, func(*types.Alert, time.Time) bool) (dispatch.AlertGroups, map[prometheus_model.Fingerprint][]string)
-type getAlertStatusFn func(prometheus_model.Fingerprint) types.AlertStatus
-type setAlertStatusFn func(prometheus_model.LabelSet)
+type (
+	groupsFn         func(func(*dispatch.Route) bool, func(*types.Alert, time.Time) bool) (dispatch.AlertGroups, map[prometheus_model.Fingerprint][]string)
+	getAlertStatusFn func(prometheus_model.Fingerprint) types.AlertStatus
+	setAlertStatusFn func(prometheus_model.LabelSet)
+)
 
 // NewAPI returns a new Alertmanager API v2
 func NewAPI(
@@ -99,9 +102,9 @@ func NewAPI(
 	}
 
 	// Load embedded swagger file.
-	swaggerSpec, err := loads.Analyzed(restapi.SwaggerJSON, "")
+	swaggerSpec, swaggerSpecAnalysis, err := getSwaggerSpec()
 	if err != nil {
-		return nil, fmt.Errorf("failed to load embedded swagger file: %v", err.Error())
+		return nil, err
 	}
 
 	// Create new service API.
@@ -111,7 +114,9 @@ func NewAPI(
 	// the API itself via RoutesHandler. See:
 	// https://github.com/go-swagger/go-swagger/issues/1779
 	openAPI.Middleware = func(b middleware.Builder) http.Handler {
-		return middleware.Spec("", swaggerSpec.Raw(), openAPI.Context().RoutesHandler(b))
+		// Manually create the context so that we can use the singleton swaggerSpecAnalysis.
+		swaggerContext := middleware.NewRoutableContextWithAnalyzedSpec(swaggerSpec, swaggerSpecAnalysis, openAPI, nil)
+		return middleware.Spec("", swaggerSpec.Raw(), swaggerContext.RoutesHandler(b))
 	}
 
 	openAPI.AlertGetAlertsHandler = alert_ops.GetAlertsHandlerFunc(api.getAlertsHandler)
@@ -225,14 +230,14 @@ func (api *API) getAlertsHandler(params alert_ops.GetAlertsParams) middleware.Re
 
 	matchers, err := parseFilter(params.Filter)
 	if err != nil {
-		level.Error(logger).Log("msg", "Failed to parse matchers", "err", err)
+		level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 		return alertgroup_ops.NewGetAlertGroupsBadRequest().WithPayload(err.Error())
 	}
 
 	if params.Receiver != nil {
 		receiverFilter, err = regexp.Compile("^(?:" + *params.Receiver + ")$")
 		if err != nil {
-			level.Error(logger).Log("msg", "Failed to compile receiver regex", "err", err)
+			level.Debug(logger).Log("msg", "Failed to compile receiver regex", "err", err)
 			return alert_ops.
 				NewGetAlertsBadRequest().
 				WithPayload(
@@ -354,7 +359,7 @@ func (api *API) getAlertGroupsHandler(params alertgroup_ops.GetAlertGroupsParams
 
 	matchers, err := parseFilter(params.Filter)
 	if err != nil {
-		level.Error(logger).Log("msg", "Failed to parse matchers", "err", err)
+		level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 		return alertgroup_ops.NewGetAlertGroupsBadRequest().WithPayload(err.Error())
 	}
 
@@ -492,7 +497,7 @@ func (api *API) getSilencesHandler(params silence_ops.GetSilencesParams) middlew
 		for _, matcherString := range params.Filter {
 			matcher, err := labels.ParseMatcher(matcherString)
 			if err != nil {
-				level.Error(logger).Log("msg", "Failed to parse matchers", "err", err)
+				level.Debug(logger).Log("msg", "Failed to parse matchers", "err", err)
 				return alert_ops.NewGetAlertsBadRequest().WithPayload(err.Error())
 			}
 
@@ -524,13 +529,11 @@ func (api *API) getSilencesHandler(params silence_ops.GetSilencesParams) middlew
 	return silence_ops.NewGetSilencesOK().WithPayload(sils)
 }
 
-var (
-	silenceStateOrder = map[types.SilenceState]int{
-		types.SilenceStateActive:  1,
-		types.SilenceStatePending: 2,
-		types.SilenceStateExpired: 3,
-	}
-)
+var silenceStateOrder = map[types.SilenceState]int{
+	types.SilenceStateActive:  1,
+	types.SilenceStatePending: 2,
+	types.SilenceStateExpired: 3,
+}
 
 // SortSilences sorts first according to the state "active, pending, expired"
 // then by end time or start time depending on the state.
@@ -672,3 +675,33 @@ func parseFilter(filter []string) ([]*labels.Matcher, error) {
 	}
 	return matchers, nil
 }
+
+var (
+	swaggerSpecCacheMx       sync.Mutex
+	swaggerSpecCache         *loads.Document
+	swaggerSpecAnalysisCache *analysis.Spec
+)
+
+// getSwaggerSpec loads and caches the swagger spec. If a cached version already exists,
+// it returns the cached one. The reason why we cache it is because some downstream projects
+// (e.g. Grafana Mimir) creates many Alertmanager instances in the same process, so they would
+// incur in a significant memory penalty if we would reload the swagger spec each time.
+func getSwaggerSpec() (*loads.Document, *analysis.Spec, error) {
+	swaggerSpecCacheMx.Lock()
+	defer swaggerSpecCacheMx.Unlock()
+
+	// Check if a cached version exists.
+	if swaggerSpecCache != nil {
+		return swaggerSpecCache, swaggerSpecAnalysisCache, nil
+	}
+
+	// Load embedded swagger file.
+	swaggerSpec, err := loads.Analyzed(restapi.SwaggerJSON, "")
+	if err != nil {
+		return nil, nil, fmt.Errorf("failed to load embedded swagger file: %w", err)
+	}
+
+	swaggerSpecCache = swaggerSpec
+	swaggerSpecAnalysisCache = analysis.New(swaggerSpec.Spec())
+	return swaggerSpec, swaggerSpecAnalysisCache, nil
+}
