@@ -14,9 +14,12 @@
 package config
 
 import (
+	"errors"
 	"strings"
 	"testing"
 
+	"github.com/stretchr/testify/require"
+
 	"gopkg.in/yaml.v2"
 )
 
@@ -57,43 +60,82 @@ headers:
 	}
 }
 
-func TestPagerdutyRoutingKeyIsPresent(t *testing.T) {
-	in := `
+func TestPagerdutyTestRoutingKey(t *testing.T) {
+	t.Run("error if no routing key or key file", func(t *testing.T) {
+		in := `
 routing_key: ''
 `
-	var cfg PagerdutyConfig
-	err := yaml.UnmarshalStrict([]byte(in), &cfg)
+		var cfg PagerdutyConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
 
-	expected := "missing service or routing key in PagerDuty config"
+		expected := "missing service or routing key in PagerDuty config"
 
-	if err == nil {
-		t.Fatalf("no error returned, expected:\n%v", expected)
-	}
-	if err.Error() != expected {
-		t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
-	}
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
+
+	t.Run("error if both routing key and key file", func(t *testing.T) {
+		in := `
+routing_key: 'xyz'
+routing_key_file: 'xyz'
+`
+		var cfg PagerdutyConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
+
+		expected := "at most one of routing_key & routing_key_file must be configured"
+
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
 }
 
-func TestPagerdutyServiceKeyIsPresent(t *testing.T) {
-	in := `
+func TestPagerdutyServiceKey(t *testing.T) {
+	t.Run("error if no service key or key file", func(t *testing.T) {
+		in := `
 service_key: ''
 `
-	var cfg PagerdutyConfig
-	err := yaml.UnmarshalStrict([]byte(in), &cfg)
+		var cfg PagerdutyConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
 
-	expected := "missing service or routing key in PagerDuty config"
+		expected := "missing service or routing key in PagerDuty config"
 
-	if err == nil {
-		t.Fatalf("no error returned, expected:\n%v", expected)
-	}
-	if err.Error() != expected {
-		t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
-	}
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
+
+	t.Run("error if both service key and key file", func(t *testing.T) {
+		in := `
+service_key: 'xyz'
+service_key_file: 'xyz'
+`
+		var cfg PagerdutyConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
+
+		expected := "at most one of service_key & service_key_file must be configured"
+
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
 }
 
 func TestPagerdutyDetails(t *testing.T) {
-
-	var tests = []struct {
+	tests := []struct {
 		in      string
 		checkFn func(map[string]string)
 	}{
@@ -137,7 +179,6 @@ details:
 	for _, tc := range tests {
 		var cfg PagerdutyConfig
 		err := yaml.UnmarshalStrict([]byte(tc.in), &cfg)
-
 		if err != nil {
 			t.Errorf("expected no error, got:%v", err)
 		}
@@ -148,6 +189,40 @@ details:
 	}
 }
 
+func TestPagerDutySource(t *testing.T) {
+	for _, tc := range []struct {
+		title string
+		in    string
+
+		expectedSource string
+	}{
+		{
+			title: "check source field is backward compatible",
+			in: `
+routing_key: 'xyz'
+client: 'alert-manager-client'
+`,
+			expectedSource: "alert-manager-client",
+		},
+		{
+			title: "check source field is set",
+			in: `
+routing_key: 'xyz'
+client: 'alert-manager-client'
+source: 'alert-manager-source'
+`,
+			expectedSource: "alert-manager-source",
+		},
+	} {
+		t.Run(tc.title, func(t *testing.T) {
+			var cfg PagerdutyConfig
+			err := yaml.UnmarshalStrict([]byte(tc.in), &cfg)
+			require.NoError(t, err)
+			require.Equal(t, tc.expectedSource, cfg.Source)
+		})
+	}
+}
+
 func TestWebhookURLIsPresent(t *testing.T) {
 	in := `{}`
 	var cfg WebhookConfig
@@ -189,7 +264,6 @@ url: 'http://example.com'
 `
 	var cfg WebhookConfig
 	err := yaml.UnmarshalStrict([]byte(in), &cfg)
-
 	if err != nil {
 		t.Fatalf("no error expected, returned:\n%v", err.Error())
 	}
@@ -205,7 +279,6 @@ http_config:
 `
 	var cfg WebhookConfig
 	err := yaml.UnmarshalStrict([]byte(in), &cfg)
-
 	if err != nil {
 		t.Fatalf("no error expected, returned:\n%v", err.Error())
 	}
@@ -219,21 +292,54 @@ http_config:
 	}
 }
 
-func TestVictorOpsRoutingKeyIsPresent(t *testing.T) {
-	in := `
+func TestVictorOpsConfiguration(t *testing.T) {
+	t.Run("valid configuration", func(t *testing.T) {
+		in := `
+routing_key: test
+api_key_file: /global_file
+`
+		var cfg VictorOpsConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
+		if err != nil {
+			t.Fatalf("no error was expected:\n%v", err)
+		}
+	})
+
+	t.Run("routing key is missing", func(t *testing.T) {
+		in := `
 routing_key: ''
 `
-	var cfg VictorOpsConfig
-	err := yaml.UnmarshalStrict([]byte(in), &cfg)
+		var cfg VictorOpsConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
 
-	expected := "missing Routing key in VictorOps config"
+		expected := "missing Routing key in VictorOps config"
 
-	if err == nil {
-		t.Fatalf("no error returned, expected:\n%v", expected)
-	}
-	if err.Error() != expected {
-		t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
-	}
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
+
+	t.Run("api_key and api_key_file both defined", func(t *testing.T) {
+		in := `
+routing_key: test
+api_key: xyz
+api_key_file: /global_file
+`
+		var cfg VictorOpsConfig
+		err := yaml.UnmarshalStrict([]byte(in), &cfg)
+
+		expected := "at most one of api_key & api_key_file must be configured"
+
+		if err == nil {
+			t.Fatalf("no error returned, expected:\n%v", expected)
+		}
+		if err.Error() != expected {
+			t.Errorf("\nexpected:\n%v\ngot:\n%v", expected, err.Error())
+		}
+	})
 }
 
 func TestVictorOpsCustomFieldsValidation(t *testing.T) {
@@ -245,7 +351,7 @@ custom_fields:
 	var cfg VictorOpsConfig
 	err := yaml.UnmarshalStrict([]byte(in), &cfg)
 
-	expected := "VictorOps config contains custom field entity_state which cannot be used as it conflicts with the fixed/static fields"
+	expected := "victorOps config contains custom field entity_state which cannot be used as it conflicts with the fixed/static fields"
 
 	if err == nil {
 		t.Fatalf("no error returned, expected:\n%v", expected)
@@ -276,7 +382,6 @@ custom_fields:
 	if val != expected {
 		t.Errorf("\nexpected custom field my_special_field value:\n%v\ngot:\n%v", expected, val)
 	}
-
 }
 
 func TestPushoverUserKeyIsPresent(t *testing.T) {
@@ -315,7 +420,7 @@ token: ''
 }
 
 func TestLoadSlackConfiguration(t *testing.T) {
-	var tests = []struct {
+	tests := []struct {
 		in       string
 		expected SlackConfig
 	}{
@@ -327,9 +432,11 @@ channel: engineering
 title_link: http://example.com/
 image_url: https://example.com/logo.png
 `,
-			expected: SlackConfig{Color: "green", Username: "mark", Channel: "engineering",
+			expected: SlackConfig{
+				Color: "green", Username: "mark", Channel: "engineering",
 				TitleLink: "http://example.com/",
-				ImageURL:  "https://example.com/logo.png"},
+				ImageURL:  "https://example.com/logo.png",
+			},
 		},
 		{
 			in: `
@@ -341,9 +448,12 @@ mrkdwn_in:
 - pretext
 - text
 `,
-			expected: SlackConfig{Color: "green", Username: "mark", Channel: "alerts",
-				MrkdwnIn: []string{"pretext", "text"}, TitleLink: "http://example.com/alert1"},
-		}}
+			expected: SlackConfig{
+				Color: "green", Username: "mark", Channel: "alerts",
+				MrkdwnIn: []string{"pretext", "text"}, TitleLink: "http://example.com/alert1",
+			},
+		},
+	}
 	for _, rt := range tests {
 		var cfg SlackConfig
 		err := yaml.UnmarshalStrict([]byte(rt.in), &cfg)
@@ -380,7 +490,7 @@ mrkdwn_in:
 }
 
 func TestSlackFieldConfigValidation(t *testing.T) {
-	var tests = []struct {
+	tests := []struct {
 		in       string
 		expected string
 	}{
@@ -449,17 +559,17 @@ fields:
   short: false
 `
 	expected := []*SlackField{
-		&SlackField{
+		{
 			Title: "first",
 			Value: "hello",
 			Short: newBoolPointer(true),
 		},
-		&SlackField{
+		{
 			Title: "second",
 			Value: "world",
 			Short: nil,
 		},
-		&SlackField{
+		{
 			Title: "third",
 			Value: "slack field test",
 			Short: newBoolPointer(false),
@@ -590,6 +700,177 @@ func TestOpsgenieTypeMatcher(t *testing.T) {
 	}
 }
 
+func TestOpsGenieConfiguration(t *testing.T) {
+	for _, tc := range []struct {
+		name string
+		in   string
+
+		err bool
+	}{
+		{
+			name: "valid configuration",
+			in: `api_key: xyz
+responders:
+- id: foo
+  type: scheDule
+- name: bar
+  type: teams
+- username: fred
+  type: USER
+api_url: http://example.com
+`,
+		},
+		{
+			name: "api_key and api_key_file both defined",
+			in: `api_key: xyz
+api_key_file: xyz
+api_url: http://example.com
+`,
+			err: true,
+		},
+		{
+			name: "invalid responder type",
+			in: `api_key: xyz
+responders:
+- id: foo
+  type: wrong
+api_url: http://example.com
+`,
+			err: true,
+		},
+		{
+			name: "missing responder field",
+			in: `api_key: xyz
+responders:
+- type: schedule
+api_url: http://example.com
+`,
+			err: true,
+		},
+		{
+			name: "valid responder type template",
+			in: `api_key: xyz
+responders:
+- id: foo
+  type: "{{/* valid comment */}}team"
+api_url: http://example.com
+`,
+		},
+		{
+			name: "invalid responder type template",
+			in: `api_key: xyz
+responders:
+- id: foo
+  type: "{{/* invalid comment }}team"
+api_url: http://example.com
+`,
+			err: true,
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			var cfg OpsGenieConfig
+
+			err := yaml.UnmarshalStrict([]byte(tc.in), &cfg)
+			if tc.err {
+				if err == nil {
+					t.Fatalf("expected error but got none")
+				}
+				return
+			}
+
+			if err != nil {
+				t.Errorf("expected no error, got %v", err)
+			}
+		})
+	}
+}
+
+func TestSNS(t *testing.T) {
+	for _, tc := range []struct {
+		in  string
+		err bool
+	}{
+		{
+			// Valid configuration without sigv4.
+			in:  `target_arn: target`,
+			err: false,
+		},
+		{
+			// Valid configuration without sigv4.
+			in:  `topic_arn: topic`,
+			err: false,
+		},
+		{
+			// Valid configuration with sigv4.
+			in: `phone_number: phone
+sigv4:
+    access_key: abc
+    secret_key: abc
+`,
+			err: false,
+		},
+		{
+			// at most one of 'target_arn', 'topic_arn' or 'phone_number' must be provided without sigv4.
+			in: `topic_arn: topic
+target_arn: target
+`,
+			err: true,
+		},
+		{
+			// at most one of 'target_arn', 'topic_arn' or 'phone_number' must be provided without sigv4.
+			in: `topic_arn: topic
+phone_number: phone
+`,
+			err: true,
+		},
+		{
+			// one of 'target_arn', 'topic_arn' or 'phone_number' must be provided without sigv4.
+			in:  "{}",
+			err: true,
+		},
+		{
+			// one of 'target_arn', 'topic_arn' or 'phone_number' must be provided with sigv4.
+			in: `sigv4:
+    access_key: abc
+    secret_key: abc
+`,
+			err: true,
+		},
+		{
+			// 'secret_key' must be provided with 'access_key'.
+			in: `topic_arn: topic
+sigv4:
+    access_key: abc
+`,
+			err: true,
+		},
+		{
+			// 'access_key' must be provided with 'secret_key'.
+			in: `topic_arn: topic
+sigv4:
+    secret_key: abc
+`,
+			err: true,
+		},
+	} {
+		t.Run("", func(t *testing.T) {
+			var cfg SNSConfig
+			err := yaml.UnmarshalStrict([]byte(tc.in), &cfg)
+			if err != nil {
+				if !tc.err {
+					t.Errorf("expecting no error, got %q", err)
+				}
+				return
+			}
+
+			if tc.err {
+				t.Logf("%#v", cfg)
+				t.Error("expecting error, got none")
+			}
+		})
+	}
+}
+
 func TestWeChatTypeMatcher(t *testing.T) {
 	good := []string{"text", "markdown"}
 	for _, g := range good {
@@ -605,6 +886,41 @@ func TestWeChatTypeMatcher(t *testing.T) {
 	}
 }
 
+func TestWebexConfiguration(t *testing.T) {
+	tc := []struct {
+		name string
+
+		in       string
+		expected error
+	}{
+		{
+			name: "with no room_id - it fails",
+			in: `
+message: xyz123
+`,
+			expected: errors.New("missing room_id on webex_config"),
+		},
+		{
+			name: "with room_id and http_config.authorization set - it succeeds",
+			in: `
+room_id: 2
+http_config:
+  authorization:
+    credentials: "xxxyyyzz"
+`,
+		},
+	}
+
+	for _, tt := range tc {
+		t.Run(tt.name, func(t *testing.T) {
+			var cfg WebexConfig
+			err := yaml.UnmarshalStrict([]byte(tt.in), &cfg)
+
+			require.Equal(t, tt.expected, err)
+		})
+	}
+}
+
 func newBoolPointer(b bool) *bool {
 	return &b
 }
