@@ -16,19 +16,21 @@ package silence
 import (
 	"bytes"
 	"fmt"
-	"io/ioutil"
 	"os"
+	"runtime"
 	"sort"
 	"testing"
 	"time"
 
+	"github.com/benbjohnson/clock"
 	"github.com/go-kit/log"
 	"github.com/matttproud/golang_protobuf_extensions/pbutil"
-	pb "github.com/prometheus/alertmanager/silence/silencepb"
-	"github.com/prometheus/alertmanager/types"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
 	"github.com/stretchr/testify/require"
+
+	pb "github.com/prometheus/alertmanager/silence/silencepb"
+	"github.com/prometheus/alertmanager/types"
 )
 
 func checkErr(t *testing.T, expected string, got error) {
@@ -80,8 +82,8 @@ func TestSilencesGC(t *testing.T) {
 	s, err := New(Options{})
 	require.NoError(t, err)
 
-	now := utcNow()
-	s.now = func() time.Time { return now }
+	s.clock = clock.NewMock()
+	now := s.nowUTC()
 
 	newSilence := func(exp time.Time) *pb.MeshSilence {
 		return &pb.MeshSilence{ExpiresAt: exp}
@@ -103,7 +105,7 @@ func TestSilencesGC(t *testing.T) {
 
 func TestSilencesSnapshot(t *testing.T) {
 	// Check whether storing and loading the snapshot is symmetric.
-	now := utcNow()
+	now := clock.NewMock().Now().UTC()
 
 	cases := []struct {
 		entries []*pb.MeshSilence
@@ -153,7 +155,7 @@ func TestSilencesSnapshot(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		f, err := ioutil.TempFile("", "snapshot")
+		f, err := os.CreateTemp("", "snapshot")
 		require.NoError(t, err, "creating temp file failed")
 
 		s1 := &Silences{st: state{}, metrics: newMetrics(nil, nil)}
@@ -179,14 +181,66 @@ func TestSilencesSnapshot(t *testing.T) {
 	}
 }
 
+// This tests a regression introduced by https://github.com/prometheus/alertmanager/pull/2689.
+func TestSilences_Maintenance_DefaultMaintenanceFuncDoesntCrash(t *testing.T) {
+	f, err := os.CreateTemp("", "snapshot")
+	require.NoError(t, err, "creating temp file failed")
+	clock := clock.NewMock()
+	s := &Silences{st: state{}, logger: log.NewNopLogger(), clock: clock, metrics: newMetrics(nil, nil)}
+	stopc := make(chan struct{})
+
+	done := make(chan struct{})
+	go func() {
+		s.Maintenance(100*time.Millisecond, f.Name(), stopc, nil)
+		close(done)
+	}()
+	runtime.Gosched()
+
+	clock.Add(100 * time.Millisecond)
+	close(stopc)
+
+	<-done
+}
+
+func TestSilences_Maintenance_SupportsCustomCallback(t *testing.T) {
+	f, err := os.CreateTemp("", "snapshot")
+	require.NoError(t, err, "creating temp file failed")
+	clock := clock.NewMock()
+	s := &Silences{st: state{}, logger: log.NewNopLogger(), clock: clock, metrics: newMetrics(nil, nil)}
+	stopc := make(chan struct{})
+
+	called := make(chan struct{}, 5)
+	go func() {
+		s.Maintenance(100*time.Millisecond, f.Name(), stopc, func() (int64, error) {
+			called <- struct{}{}
+			return 0, nil
+		})
+		close(called)
+	}()
+	runtime.Gosched()
+
+	clock.Add(100 * time.Millisecond)
+
+	// Stop the maintenance loop. We should get exactly one more execution of the maintenance func.
+	close(stopc)
+	calls := 0
+	for range called {
+		calls++
+	}
+
+	require.EqualValues(t, 2, calls)
+}
+
 func TestSilencesSetSilence(t *testing.T) {
 	s, err := New(Options{
 		Retention: time.Minute,
 	})
 	require.NoError(t, err)
 
-	now := utcNow()
-	nowpb := now
+	clock := clock.NewMock()
+	s.clock = clock
+
+	nowpb := s.nowUTC()
 
 	sil := &pb.Silence{
 		Id:       "some_id",
@@ -198,11 +252,11 @@ func TestSilencesSetSilence(t *testing.T) {
 	want := state{
 		"some_id": &pb.MeshSilence{
 			Silence:   sil,
-			ExpiresAt: now.Add(time.Minute),
+			ExpiresAt: nowpb.Add(time.Minute),
 		},
 	}
 
-	done := make(chan bool)
+	done := make(chan struct{})
 	s.broadcast = func(b []byte) {
 		var e pb.MeshSilence
 		r := bytes.NewReader(b)
@@ -213,18 +267,16 @@ func TestSilencesSetSilence(t *testing.T) {
 		close(done)
 	}
 
-	// setSilence() is always called with s.mtx locked()
-	go func() {
+	// setSilence() is always called with s.mtx locked() in the application code
+	func() {
 		s.mtx.Lock()
-		require.NoError(t, s.setSilence(sil, now))
-		s.mtx.Unlock()
+		defer s.mtx.Unlock()
+		require.NoError(t, s.setSilence(sil, nowpb))
 	}()
 
-	// GossipBroadcast is called in a goroutine.
-	select {
-	case <-done:
-	case <-time.After(1 * time.Second):
-		t.Fatal("GossipBroadcast was not called")
+	// Ensure broadcast was called.
+	if _, isOpen := <-done; isOpen {
+		t.Fatal("broadcast was not called")
 	}
 
 	require.Equal(t, want, s.st, "Unexpected silence state")
@@ -236,15 +288,15 @@ func TestSilenceSet(t *testing.T) {
 	})
 	require.NoError(t, err)
 
-	now := utcNow()
-	now1 := now
-	s.now = func() time.Time { return now }
+	clock := clock.NewMock()
+	s.clock = clock
+	start1 := s.nowUTC()
 
 	// Insert silence with fixed start time.
 	sil1 := &pb.Silence{
 		Matchers: []*pb.Matcher{{Name: "a", Pattern: "b"}},
-		StartsAt: now.Add(2 * time.Minute),
-		EndsAt:   now.Add(5 * time.Minute),
+		StartsAt: start1.Add(2 * time.Minute),
+		EndsAt:   start1.Add(5 * time.Minute),
 	}
 	id1, err := s.Set(sil1)
 	require.NoError(t, err)
@@ -255,22 +307,22 @@ func TestSilenceSet(t *testing.T) {
 			Silence: &pb.Silence{
 				Id:        id1,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
-				StartsAt:  now1.Add(2 * time.Minute),
-				EndsAt:    now1.Add(5 * time.Minute),
-				UpdatedAt: now1,
+				StartsAt:  start1.Add(2 * time.Minute),
+				EndsAt:    start1.Add(5 * time.Minute),
+				UpdatedAt: start1,
 			},
-			ExpiresAt: now1.Add(5*time.Minute + s.retention),
+			ExpiresAt: start1.Add(5*time.Minute + s.retention),
 		},
 	}
 	require.Equal(t, want, s.st, "unexpected state after silence creation")
 
 	// Insert silence with unset start time. Must be set to now.
-	now = now.Add(time.Minute)
-	now2 := now
+	clock.Add(time.Minute)
+	start2 := s.nowUTC()
 
 	sil2 := &pb.Silence{
 		Matchers: []*pb.Matcher{{Name: "a", Pattern: "b"}},
-		EndsAt:   now.Add(1 * time.Minute),
+		EndsAt:   start2.Add(1 * time.Minute),
 	}
 	id2, err := s.Set(sil2)
 	require.NoError(t, err)
@@ -282,21 +334,21 @@ func TestSilenceSet(t *testing.T) {
 			Silence: &pb.Silence{
 				Id:        id2,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
-				StartsAt:  now2,
-				EndsAt:    now2.Add(1 * time.Minute),
-				UpdatedAt: now2,
+				StartsAt:  start2,
+				EndsAt:    start2.Add(1 * time.Minute),
+				UpdatedAt: start2,
 			},
-			ExpiresAt: now2.Add(1*time.Minute + s.retention),
+			ExpiresAt: start2.Add(1*time.Minute + s.retention),
 		},
 	}
 	require.Equal(t, want, s.st, "unexpected state after silence creation")
 
 	// Overwrite silence 2 with new end time.
-	now = now.Add(time.Minute)
-	now3 := now
+	clock.Add(time.Minute)
+	start3 := s.nowUTC()
 
 	sil3 := cloneSilence(sil2)
-	sil3.EndsAt = now.Add(100 * time.Minute)
+	sil3.EndsAt = start3.Add(100 * time.Minute)
 
 	id3, err := s.Set(sil3)
 	require.NoError(t, err)
@@ -308,24 +360,25 @@ func TestSilenceSet(t *testing.T) {
 			Silence: &pb.Silence{
 				Id:        id2,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
-				StartsAt:  now2,
-				EndsAt:    now3.Add(100 * time.Minute),
-				UpdatedAt: now3,
+				StartsAt:  start2,
+				EndsAt:    start3.Add(100 * time.Minute),
+				UpdatedAt: start3,
 			},
-			ExpiresAt: now3.Add(100*time.Minute + s.retention),
+			ExpiresAt: start3.Add(100*time.Minute + s.retention),
 		},
 	}
 	require.Equal(t, want, s.st, "unexpected state after silence creation")
 
-	// Update silence 2 with new matcher expires it and creates a new one.
-	now = now.Add(time.Minute)
-	now4 := now
+	// Update this silence again with new matcher. This expires it and creates a new one.
+	clock.Add(time.Minute)
+	start4 := s.nowUTC()
 
 	sil4 := cloneSilence(sil3)
 	sil4.Matchers = []*pb.Matcher{{Name: "a", Pattern: "c"}}
 
 	id4, err := s.Set(sil4)
 	require.NoError(t, err)
+	// This new silence gets a new id.
 	require.NotEqual(t, id2, id4)
 
 	want = state{
@@ -334,32 +387,32 @@ func TestSilenceSet(t *testing.T) {
 			Silence: &pb.Silence{
 				Id:        id2,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
-				StartsAt:  now2,
-				EndsAt:    now4,
-				UpdatedAt: now4,
+				StartsAt:  start2,
+				EndsAt:    start4, // Expired
+				UpdatedAt: start4,
 			},
-			ExpiresAt: now4.Add(s.retention),
+			ExpiresAt: start4.Add(s.retention),
 		},
 		id4: &pb.MeshSilence{
 			Silence: &pb.Silence{
 				Id:        id4,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "c"}},
-				StartsAt:  now4,
-				EndsAt:    now3.Add(100 * time.Minute),
-				UpdatedAt: now4,
+				StartsAt:  start4,
+				EndsAt:    start3.Add(100 * time.Minute),
+				UpdatedAt: start4,
 			},
-			ExpiresAt: now3.Add(100*time.Minute + s.retention),
+			ExpiresAt: start3.Add(100*time.Minute + s.retention),
 		},
 	}
 	require.Equal(t, want, s.st, "unexpected state after silence creation")
 
-	// Re-create expired silence.
-	now = now.Add(time.Minute)
-	now5 := now
+	// Re-create the silence that just expired.
+	clock.Add(time.Minute)
+	start5 := s.nowUTC()
 
 	sil5 := cloneSilence(sil3)
-	sil5.StartsAt = now
-	sil5.EndsAt = now.Add(5 * time.Minute)
+	sil5.StartsAt = start1
+	sil5.EndsAt = start1.Add(5 * time.Minute)
 
 	id5, err := s.Set(sil5)
 	require.NoError(t, err)
@@ -373,11 +426,62 @@ func TestSilenceSet(t *testing.T) {
 			Silence: &pb.Silence{
 				Id:        id5,
 				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
-				StartsAt:  now5,
-				EndsAt:    now5.Add(5 * time.Minute),
-				UpdatedAt: now5,
+				StartsAt:  start5, // New silences have their start time set to "now" when created.
+				EndsAt:    start1.Add(5 * time.Minute),
+				UpdatedAt: start5,
 			},
-			ExpiresAt: now5.Add(5*time.Minute + s.retention),
+			ExpiresAt: start1.Add(5*time.Minute + s.retention),
+		},
+	}
+	require.Equal(t, want, s.st, "unexpected state after silence creation")
+}
+
+func TestSetActiveSilence(t *testing.T) {
+	s, err := New(Options{
+		Retention: time.Hour,
+	})
+	require.NoError(t, err)
+
+	clock := clock.NewMock()
+	s.clock = clock
+	now := clock.Now()
+
+	startsAt := now.Add(-1 * time.Minute)
+	endsAt := now.Add(5 * time.Minute)
+	// Insert silence with fixed start time.
+	sil1 := &pb.Silence{
+		Matchers: []*pb.Matcher{{Name: "a", Pattern: "b"}},
+		StartsAt: startsAt,
+		EndsAt:   endsAt,
+	}
+	id1, _ := s.Set(sil1)
+
+	// Update silence with 2 extra nanoseconds so the "seconds" part should not change
+
+	newStartsAt := now.Add(2 * time.Nanosecond)
+	newEndsAt := endsAt.Add(2 * time.Minute)
+
+	sil2 := cloneSilence(sil1)
+	sil2.Id = id1
+	sil2.StartsAt = newStartsAt
+	sil2.EndsAt = newEndsAt
+
+	clock.Add(time.Minute)
+	now = s.nowUTC()
+	id2, err := s.Set(sil2)
+	require.NoError(t, err)
+	require.Equal(t, id1, id2)
+
+	want := state{
+		id2: &pb.MeshSilence{
+			Silence: &pb.Silence{
+				Id:        id1,
+				Matchers:  []*pb.Matcher{{Name: "a", Pattern: "b"}},
+				StartsAt:  newStartsAt,
+				EndsAt:    newEndsAt,
+				UpdatedAt: now,
+			},
+			ExpiresAt: newEndsAt.Add(s.retention),
 		},
 	}
 	require.Equal(t, want, s.st, "unexpected state after silence creation")
@@ -387,8 +491,8 @@ func TestSilencesSetFail(t *testing.T) {
 	s, err := New(Options{})
 	require.NoError(t, err)
 
-	now := utcNow()
-	s.now = func() time.Time { return now }
+	clock := clock.NewMock()
+	s.clock = clock
 
 	cases := []struct {
 		s   *pb.Silence
@@ -409,7 +513,7 @@ func TestSilencesSetFail(t *testing.T) {
 }
 
 func TestQState(t *testing.T) {
-	now := utcNow()
+	now := time.Now().UTC()
 
 	cases := []struct {
 		sil    *pb.Silence
@@ -620,7 +724,7 @@ func (s silencesByID) Swap(i, j int)      { s[i], s[j] = s[j], s[i] }
 func (s silencesByID) Less(i, j int) bool { return s[i].Id < s[j].Id }
 
 func TestSilenceCanUpdate(t *testing.T) {
-	now := utcNow()
+	now := time.Now().UTC()
 
 	cases := []struct {
 		a, b *pb.Silence
@@ -749,8 +853,9 @@ func TestSilenceExpire(t *testing.T) {
 	s, err := New(Options{Retention: time.Hour})
 	require.NoError(t, err)
 
-	now := time.Now()
-	s.now = func() time.Time { return now }
+	clock := clock.NewMock()
+	s.clock = clock
+	now := s.nowUTC()
 
 	m := &pb.Matcher{Type: pb.Matcher_EQUAL, Name: "a", Pattern: "b"}
 
@@ -789,9 +894,7 @@ func TestSilenceExpire(t *testing.T) {
 	require.NoError(t, s.Expire("pending"))
 	require.NoError(t, s.Expire("active"))
 
-	err = s.Expire("expired")
-	require.Error(t, err)
-	require.Contains(t, err.Error(), "already expired")
+	require.NoError(t, s.Expire("expired"))
 
 	sil, err := s.QueryOne(QIDs("pending"))
 	require.NoError(t, err)
@@ -804,7 +907,7 @@ func TestSilenceExpire(t *testing.T) {
 	}, sil)
 
 	// Let time pass...
-	s.now = func() time.Time { return now.Add(time.Second) }
+	clock.Add(time.Second)
 
 	count, err = s.CountState(types.SilenceStatePending)
 	require.NoError(t, err)
@@ -838,19 +941,18 @@ func TestSilenceExpire(t *testing.T) {
 		EndsAt:    now.Add(-time.Minute),
 		UpdatedAt: now.Add(-time.Hour),
 	}, sil)
-
 }
 
 // TestSilenceExpireWithZeroRetention covers the problem that, with zero
 // retention time, a silence explicitly set to expired will also immediately
 // expire from the silence storage.
 func TestSilenceExpireWithZeroRetention(t *testing.T) {
-	s, err := New(Options{})
+	s, err := New(Options{Retention: 0})
 	require.NoError(t, err)
 
-	now := time.Now()
-	// But don't set s.now here because we need a changing time to
-	// demonstrate expiration from the silence storage.
+	clock := clock.NewMock()
+	s.clock = clock
+	now := s.nowUTC()
 
 	m := &pb.Matcher{Type: pb.Matcher_EQUAL, Name: "a", Pattern: "b"}
 
@@ -882,25 +984,37 @@ func TestSilenceExpireWithZeroRetention(t *testing.T) {
 	require.NoError(t, err)
 	require.Equal(t, 1, count)
 
+	count, err = s.CountState(types.SilenceStateActive)
+	require.NoError(t, err)
+	require.Equal(t, 1, count)
+
 	count, err = s.CountState(types.SilenceStateExpired)
 	require.NoError(t, err)
 	require.Equal(t, 1, count)
 
+	// Advance time. The silence state management code uses update time when
+	// merging, and the logic is "first write wins". So we must advance the clock
+	// one tick for updates to take effect.
+	clock.Add(1 * time.Millisecond)
+
 	require.NoError(t, s.Expire("pending"))
 	require.NoError(t, s.Expire("active"))
+	require.NoError(t, s.Expire("expired"))
 
-	err = s.Expire("expired")
-	require.Error(t, err)
-	require.Contains(t, err.Error(), "already expired")
-
-	_, err = s.QueryOne(QIDs("pending"))
-	require.NoError(t, err)
-	require.Equal(t, 1, count)
+	// Advance time again. Despite what the function name says, s.Expire() does
+	// not expire a silence. It sets the silence to EndAt the current time. This
+	// means that the silence is active immediately after calling Expire.
+	clock.Add(1 * time.Millisecond)
 
+	// Verify all silences have expired.
 	count, err = s.CountState(types.SilenceStatePending)
 	require.NoError(t, err)
 	require.Equal(t, 0, count)
 
+	count, err = s.CountState(types.SilenceStateActive)
+	require.NoError(t, err)
+	require.Equal(t, 0, count)
+
 	count, err = s.CountState(types.SilenceStateExpired)
 	require.NoError(t, err)
 	require.Equal(t, 3, count)
@@ -910,8 +1024,9 @@ func TestSilencer(t *testing.T) {
 	ss, err := New(Options{Retention: time.Hour})
 	require.NoError(t, err)
 
-	now := time.Now()
-	ss.now = func() time.Time { return now }
+	clock := clock.NewMock()
+	ss.clock = clock
+	now := ss.nowUTC()
 
 	m := types.NewMarker(prometheus.NewRegistry())
 	s := NewSilencer(ss, m, log.NewNopLogger())
@@ -937,7 +1052,9 @@ func TestSilencer(t *testing.T) {
 
 	require.True(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert silenced by matching silence")
 
-	now = now.Add(time.Hour) // One hour passes, silence expires.
+	// One hour passes, silence expires.
+	clock.Add(time.Hour)
+	now = ss.nowUTC()
 
 	require.False(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert not silenced by expired silence")
 
@@ -952,7 +1069,9 @@ func TestSilencer(t *testing.T) {
 
 	require.False(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert not silenced by future silence")
 
-	now = now.Add(2 * time.Hour) // Two hours pass, silence becomes active.
+	// Two hours pass, silence becomes active.
+	clock.Add(2 * time.Hour)
+	now = ss.nowUTC()
 
 	// Exposes issue #2426.
 	require.True(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert silenced by activated silence")
@@ -967,7 +1086,8 @@ func TestSilencer(t *testing.T) {
 	// Note that issue #2426 doesn't apply anymore because we added a new silence.
 	require.True(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert still silenced by activated silence")
 
-	now = now.Add(2 * time.Hour) // Two hours pass, first silence expires, overlapping second silence becomes active.
+	// Two hours pass, first silence expires, overlapping second silence becomes active.
+	clock.Add(2 * time.Hour)
 
 	// Another variant of issue #2426 (overlapping silences).
 	require.True(t, s.Mutes(model.LabelSet{"foo": "bar"}), "expected alert silenced by activated second silence")
@@ -1045,13 +1165,13 @@ func TestValidateMatcher(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		checkErr(t, c.err, validateMatcher(c.m))
+		checkErr(t, c.err, ValidateMatcher(c.m))
 	}
 }
 
 func TestValidateSilence(t *testing.T) {
 	var (
-		now            = utcNow()
+		now            = time.Now().UTC()
 		zeroTimestamp  = time.Time{}
 		validTimestamp = now
 	)
@@ -1063,7 +1183,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    validTimestamp,
@@ -1075,7 +1195,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    validTimestamp,
@@ -1097,8 +1217,8 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
-					&pb.Matcher{Name: "00", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
+					{Name: "00", Pattern: "b"},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    validTimestamp,
@@ -1110,8 +1230,8 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: ""},
-					&pb.Matcher{Name: "b", Pattern: ".*", Type: pb.Matcher_REGEXP},
+					{Name: "a", Pattern: ""},
+					{Name: "b", Pattern: ".*", Type: pb.Matcher_REGEXP},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    validTimestamp,
@@ -1123,7 +1243,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  now,
 				EndsAt:    now.Add(-time.Second),
@@ -1135,7 +1255,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  zeroTimestamp,
 				EndsAt:    validTimestamp,
@@ -1147,7 +1267,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    zeroTimestamp,
@@ -1159,7 +1279,7 @@ func TestValidateSilence(t *testing.T) {
 			s: &pb.Silence{
 				Id: "some_id",
 				Matchers: []*pb.Matcher{
-					&pb.Matcher{Name: "a", Pattern: "b"},
+					{Name: "a", Pattern: "b"},
 				},
 				StartsAt:  validTimestamp,
 				EndsAt:    validTimestamp,
@@ -1174,7 +1294,7 @@ func TestValidateSilence(t *testing.T) {
 }
 
 func TestStateMerge(t *testing.T) {
-	now := utcNow()
+	now := time.Now().UTC()
 
 	// We only care about key names and timestamps for the
 	// merging logic.
@@ -1223,7 +1343,7 @@ func TestStateMerge(t *testing.T) {
 
 func TestStateCoding(t *testing.T) {
 	// Check whether encoding and decoding the data is symmetric.
-	now := utcNow()
+	now := time.Now().UTC()
 
 	cases := []struct {
 		entries []*pb.MeshSilence
@@ -1303,8 +1423,9 @@ func benchmarkSilencesQuery(b *testing.B, numSilences int) {
 	s, err := New(Options{})
 	require.NoError(b, err)
 
-	now := time.Now()
-	s.now = func() time.Time { return now }
+	clock := clock.NewMock()
+	s.clock = clock
+	now := clock.Now()
 
 	lset := model.LabelSet{"aaaa": "AAAA", "bbbb": "BBBB", "cccc": "CCCC"}
 
@@ -1322,8 +1443,8 @@ func benchmarkSilencesQuery(b *testing.B, numSilences int) {
 		s.st[id] = &pb.MeshSilence{Silence: &pb.Silence{
 			Id: id,
 			Matchers: []*pb.Matcher{
-				&pb.Matcher{Type: pb.Matcher_REGEXP, Name: "aaaa", Pattern: patA},
-				&pb.Matcher{Type: pb.Matcher_REGEXP, Name: "bbbb", Pattern: patB},
+				{Type: pb.Matcher_REGEXP, Name: "aaaa", Pattern: patA},
+				{Type: pb.Matcher_REGEXP, Name: "bbbb", Pattern: patB},
 			},
 			StartsAt:  now.Add(-time.Minute),
 			EndsAt:    now.Add(time.Hour),
